from __future__ import annotations

from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from ..client import DSGCoreClient


class SubclientBase:
    def __init__(self, client: "DSGCoreClient"):
        self._client = client
